# anyascii is a Wagtail dependency
from anyascii import anyascii

from django.utils.text import slugify


def clean_form_field_name(label):
    # anyascii will return an ascii string while slugify wants a
    # unicode string on the other hand, slugify returns a safe-string
    # which will be converted to a normal str
    return str(slugify(str(anyascii(label))))



from wagtail import blocks
from .conditional_block import ConditionalBlock


class BaseFieldBlock(blocks.StructBlock):

    label = blocks.CharBlock(
        max_length=255,
        required=True,
        help_text='The label of the form field.'
    )

    required = blocks.BooleanBlock(
        default=False,
        required=False,
        help_text='Tick this box to make this a required field.'
    )

    display_checkbox_label = blocks.BooleanBlock(
        default=False,
        required=False,
        help_text='Do you want the checkbox label to display? If not you should populate help_text.'
    )

    choices = blocks.ListBlock(
        blocks.TextBlock(
            required=False,
        ),
    )

    empty_label = blocks.CharBlock(
        max_length=255,
        required=False,
    )

    max_length = blocks.IntegerBlock(
        required=False,
        help_text='Set a maximum length for this field. e.g. 100'
    )

    default_value = blocks.CharBlock(
        max_length=255,
        required=False,
        help_text='Set a default value for this field.'
    )

    # placeholder = blocks.CharBlock(
    #     max_length=255,
    #     required=False,
    #     help_text='Set a placeholder for the field.'
    # )

    help_text = blocks.CharBlock(
        max_length=255,
        required=False,
        help_text='Text to assist the user in populating this field.'
    )

    html = blocks.RichTextBlock(
        required=False,
    )

    display_side_by_side = blocks.BooleanBlock(
        help_text='Display these items side by side?',
        required=False,
    )

    rules = ConditionalBlock(
        required=False,
        help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.'
    )



import wagtail_advanced_form_builder.constants as consts

from wagtail import blocks

from .condition_block import ConditionBlock
from .blank_condition_block import BlankConditionBlock


class ConditionalBlock(blocks.StructBlock):

    action = blocks.ChoiceBlock(
        choices=consts.FIELD_ACTION_CHOICES,
        default=consts.FIELD_ACTION_SHOW,
        help_text='What conditional action would you like to perform on this field?',
        required=True,
    )

    conditions = blocks.StreamBlock(
        [
            ('condition', ConditionBlock()),
            ('blank_condition', BlankConditionBlock()),
        ],
        required=False,
    )

    class Meta:
        icon = 'cog'
        form_classname = 'waf--form-rules waf--field-content'



from django import template

register = template.Library()


@register.filter('fieldtype')
def fieldtype(field):
    return field.field.widget.__class__.__name__


@register.inclusion_tag('wagtail_advanced_form_builder/tags/form_field.html', takes_context=True)
def form_field(context, field):
    return {
        'field': field,
    }


@register.simple_tag
def wrap_form_builder_field(field):
    try:
        field_type = fieldtype(field)
        field_class = ''
        attrs = field.field.widget.attrs
        attrs['class'] = field_class
        field.field.widget.attrs = attrs

    except Exception as e:
        return field

    return field



from django.forms import RadioSelect


class SideBySideRadioSelectWidget(RadioSelect):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.display_side_by_side = True



import json

from django.db import models
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _

from wagtail import VERSION as WAGTAIL_VERSION
if WAGTAIL_VERSION >= (3, 0):
    from wagtail.admin.panels import FieldPanel, MultiFieldPanel
else:
    from wagtail.admin.edit_handlers import StreamFieldPanel, FieldPanel, MultiFieldPanel
from wagtail.fields import RichTextField, StreamField
from wagtail.rich_text import RichText

from wagtail_advanced_form_builder.blocks.fields import SingleLineFieldBlock
from wagtail_advanced_form_builder.forms import AdvancedFormBuilder
from .form_field import FormField

import wagtail_advanced_form_builder.constants as consts
from wagtail_advanced_form_builder.utils import clean_form_field_name
from ..blocks.fields.checkbox_field_block import CheckboxFieldBlock
from ..blocks.fields.checkboxes_field_block import CheckboxesFieldBlock
from ..blocks.fields.dropdown_field_block import DropdownFieldBlock
from ..blocks.fields.email_field_block import EmailFieldBlock
from ..blocks.fields.hidden_field_block import HiddenFieldBlock
from ..blocks.fields.html_field_block import HTMLFieldBlock
from ..blocks.fields.multi_line_field_block import MultiLineFieldBlock
from ..blocks.fields.multiselect_field_block import MultiSelectFieldBlock
from ..blocks.fields.number_field_block import NumberFieldBlock
from ..blocks.fields.radio_field_block import RadioFieldBlock
from ..blocks.fields.url_field_block import URLFieldBlock


def parse_condition_value(value):

    if value is False or value is None:
        return ''

    if value is True:
        return 'on'

    if isinstance(value, list):
        if len(value) == 0:
            return ''
        else:
            return ','.join(value)

    return value

def conditions_passed(condition_rule, condition_value, condition_field_value):

    condition_field_value = parse_condition_value(condition_field_value)

    if condition_rule == consts.FIELD_RULE_IS:
        if condition_field_value != condition_value:
            return False

    if condition_rule == consts.FIELD_RULE_IS_NOT:
        if condition_field_value == condition_value:
            return False

    if condition_rule == consts.FIELD_RULE_IS_BLANK:
        if condition_field_value != '':
            return False

    if condition_rule == consts.FIELD_RULE_IS_NOT_BLANK:
        if condition_field_value == '':
            return False

    if condition_rule == consts.FIELD_RULE_GREATER_THAN:
        try:
            condition_field_value = float(condition_field_value)
            condition_value = float(condition_value)
            if condition_field_value <= condition_value:
                return False
        except ValueError:
            pass

    if condition_rule == consts.FIELD_RULE_GREATER_THAN_OR_EQUAL:
        try:
            condition_field_value = float(condition_field_value)
            condition_value = float(condition_value)
            if condition_field_value < condition_value:
                return False
        except ValueError:
            pass

    if condition_rule == consts.FIELD_RULE_LESS_THAN:
        try:
            condition_field_value = float(condition_field_value)
            condition_value = float(condition_value)
            if condition_field_value >= condition_value:
                return False
        except ValueError:
            pass

    if condition_rule == consts.FIELD_RULE_LESS_THAN_OR_EQUAL:
        try:
            condition_field_value = float(condition_field_value)
            condition_value = float(condition_value)
            if condition_field_value > condition_value:
                return False
        except ValueError:
            pass

    if condition_rule == consts.FIELD_RULE_CONTAINS:
        if condition_value not in condition_field_value:
            return False

    if condition_rule == consts.FIELD_RULE_STARTS_WITH:
        if not condition_field_value.startsWith(condition_value):
            return False

    if condition_rule == consts.FIELD_RULE_ENDS_WITH:
        if not condition_field_value.endsWith(condition_value):
            return False

    return True


def clean_form(self):
    """
    Override the form builder form clean method to add validation to check for existing
    submissions
    :param self:
    :return:
    """

    for condition in self.conditional_rules:

        all_conditions_passed = True

        # Only check if the field is required
        field_required = condition['required']
        if field_required:
            action = condition['action']

            for cond in condition['conditions']:
                condition_rule = cond['rule']
                condition_value = cond['value']
                condition_field_value = self.cleaned_data.get(cond['field_name'], None)

                all_conditions_passed = conditions_passed(condition_rule, condition_value, condition_field_value)
                if not all_conditions_passed:
                    break

            if not all_conditions_passed and action == consts.FIELD_ACTION_SHOW:
                if condition['field_name'] in self._errors:
                    del self._errors[condition['field_name']]
            elif all_conditions_passed and action == consts.FIELD_ACTION_HIDE:
                if condition['field_name'] in self._errors:
                    del self._errors[condition['field_name']]

    return self.cleaned_data


class AbstractAdvancedFormMixin(models.Model):

    landing_page_template = 'wagtail_advanced_form_builder/page/form_complete.html'

    form_builder = AdvancedFormBuilder

    form_field = FormField

    form = StreamField(
        [
            (consts.FIELD_TYPE_SINGLE_LINE, SingleLineFieldBlock(label=_('Basic field'))),
            (consts.FIELD_TYPE_EMAIL, EmailFieldBlock(label=_('Email field'))),
            (consts.FIELD_TYPE_HTML, HTMLFieldBlock(label=_('HTML'))),
            (consts.FIELD_TYPE_DROPDOWN, DropdownFieldBlock(label=_('Dropdown field'))),
            (consts.FIELD_TYPE_RADIO, RadioFieldBlock(label=_('Radio field'))),
            (consts.FIELD_TYPE_CHECKBOXES, CheckboxesFieldBlock(label=_('Checkboxes field'))),
            (consts.FIELD_TYPE_CHECKBOX, CheckboxFieldBlock(label=_('Checkbox field'))),
            (consts.FIELD_TYPE_MULTI_SELECT, MultiSelectFieldBlock(label=_('Multiselect field'))),
            (consts.FIELD_TYPE_MULTI_LINE, MultiLineFieldBlock(label=_('Multiline field'))),
            (consts.FIELD_TYPE_URL, URLFieldBlock(label=_('URL field'))),
            (consts.FIELD_TYPE_HIDDEN, HiddenFieldBlock(label=_('Hidden field'))),
            (consts.FIELD_TYPE_NUMBER, NumberFieldBlock(label=_('Number field'))),
        ],
        default=None,
        null=True,
        blank=False,
        help_text=_('Add fields to the form'),
        use_json_field=True,
    )

    thanks_page_title = models.CharField(
        max_length=150,
        default=None,
        null=True,
        blank=True,
        help_text=_('The title for the thanks page. Defaults to the normal page title if left blank.')
    )

    thanks_page_content = RichTextField(
        default=None,
        null=True,
        blank=False,
        help_text=_('Content to display on the form thank you page.')
    )

    submit_button_text = models.CharField(
        max_length=30,
        default=_('Submit'),
        help_text=_('The text to display on the form submission button'),
        blank=False,
    )

    use_browser_validation = models.BooleanField(
        blank=True,
        default=False,
        help_text=_('Tick this to use the in-built browser validation on fields')
    )

    content_panels = [
        FieldPanel('form') if WAGTAIL_VERSION >= (3, 0) else StreamFieldPanel('form'),
        FieldPanel('submit_button_text'),
        MultiFieldPanel(
            [
                FieldPanel('thanks_page_title'),
                FieldPanel('thanks_page_content'),
            ],
            heading='Thanks page'
        ),
    ]

    settings_panels = [
        FieldPanel('use_browser_validation'),
    ]

    def get_template(self, request, *args, **kwargs):
        return 'wagtail_advanced_form_builder/page/form.html'

    def get_conditional_rules(self):
        """
        Retrieves the conditional rules that have been applied to this form within the CMS and exposes
        them to a dictionary which can be used to drive the front end form and to the form validator.
        :return: Dict
        """
        conditional_rules = []

        for field in self.form.get_prep_value():

            rules = field['value'].get('rules', None)
            if rules:
                field_id = field['value'].get('field_id', None)
                if field_id:
                    rules['field_name'] = field_id
                else:
                    rules['field_name'] = clean_form_field_name(field['value']['label'])
                rules['required'] = field['value'].get('required', False)
                rules['field_type'] = field.get('type', None)
                conditions = rules.get('conditions', None)
                if len(conditions):
                    for condition in conditions:
                        del(condition['id'])
                        del(condition['type'])
                        condition['field_name'] = clean_form_field_name(condition['value']['field_name'])
                        condition['rule'] = condition['value']['rule']
                        condition['value'] = condition['value'].get('value', None)

                    conditional_rules.append(rules)

        return conditional_rules

    def get_form(self, *args, **kwargs):

        """
        Overriden get_form method to allow for a custom validator on the form
        :param args:
        :param kwargs:
        :return:
        """

        # Call AbstractForm get_form method to retrieve form normally
        form = super().get_form(*args, **kwargs)

        # Dynamically override the clean method of the form builder form with a descriptor method
        form.clean = clean_form.__get__(form)

        # Attach anything else you need to the form in order to do your validation
        form.conditional_rules = self.get_conditional_rules()

        return form

    def get_data_fields(self):
        """
        Returns a list of tuples with (field_name, field_label).
        """
        data_fields = [
            ('submit_time', _('Submission date')),
        ]
        data_fields += [
            (field.clean_name, field.label)
            for field in self.get_form_fields(exclude_html=True)
        ]

        return data_fields

    def get_form_fields(self, exclude_html=False):
        """
        Get the form fields for this form by iterating over the streamfield block form fields and converting them
        into
        :param exclude_html:
        :return:
        """
        fields = []
        for field in self.form.get_prep_value():

            field_type = field['type']
            if exclude_html and field_type == consts.FIELD_TYPE_HTML:
                continue

            html_value = field['value'].get('html', None)
            if html_value:
                html_value = RichText(html_value)

            rule_action = ''
            rules = field['value'].get('rules', None)
            if rules:
                conditions = rules.get('conditions', None)
                if len(conditions):
                    rule_action = rules.get('action')

            form_field = self.form_field(
                field_type=field['type'],
                label=field['value']['label'],
                required=field['value'].get('required', False),
                choices=field['value'].get('choices', None),
                help_text=field['value'].get('help_text', None),
                default_value=field['value'].get('default_value', None),
                empty_label=field['value'].get('empty_label', None),
                max_length=field['value'].get('max_length', None),
                display_side_by_side=field['value'].get('display_side_by_side', False),
                display_checkbox_label=field['value'].get('display_checkbox_label', False),
                html_value=html_value,
                rule_action=rule_action,
            )

            fields.append(form_field)

        return fields

    def get_preview_context(self, request, mode_name):
        """
        Returns a context dictionary for use in templates for previewing this object.
        """
        context = super().get_preview_context(request, mode_name)
        form = self.get_form(page=self, user=request.user)
        context['form'] = form
        if form:
            context['conditional_rules'] = json.dumps(form.conditional_rules)

        return context

    def serve(self, request, *args, **kwargs):
        """
        Process the form submission. Overriding the abstract class method so we can redirect to the thanks page as opposed
        to just outputting it.
        :param request:
        :param args:
        :param kwargs:
        :return:
        """

        template = self.get_template(request)

        if request.method == 'POST':

            form = self.get_form(request.POST, page=self, user=request.user)

            if form.is_valid():
                self.process_form_submission(form)
                return HttpResponseRedirect(self.url + '?thank=you')

        else:

            thanks = request.GET.get('thank', False)
            if thanks:
                form = None
                template = self.get_landing_page_template(request)
                if self.thanks_page_title:
                    self.title = self.thanks_page_title
            else:
                form = self.get_form(page=self, user=request.user)

        context = self.get_context(request)
        context['form'] = form
        if form:
            context['conditional_rules'] = json.dumps(form.conditional_rules)

        return render(
            request,
            template,
            context
        )

    class Meta:
        abstract = True



from .base_field_block import BaseFieldBlock
from .blank_condition_block import BlankConditionBlock
from .condition_block import ConditionBlock
from .conditional_block import ConditionalBlock
from .single_line_field_block import SingleLineFieldBlock



from wagtail.models import Page

from .abstract_advanced_email_form import AbstractAdvancedEmailForm


class EmailFormPage(AbstractAdvancedEmailForm):

    content_panels = Page.content_panels + AbstractAdvancedEmailForm.content_panels

    settings_panels = Page.settings_panels + AbstractAdvancedEmailForm.settings_panels



import wagtail_advanced_form_builder.constants as consts

from wagtail import blocks

from .condition_block import ConditionBlock


class BlankConditionBlock(ConditionBlock):

    rule = blocks.ChoiceBlock(
        choices=consts.FIELD_RULE_CHOICES_BLANK,
        default=consts.FIELD_RULE_IS_BLANK,
    )

    value = None

    class Meta:
        icon = 'cog'




from .base_field_block import BaseFieldBlock


class SingleLineFieldBlock(BaseFieldBlock):

    html = None

    choices = None

    empty_label = None

    display_side_by_side = None

    display_checkbox_label = None

    class Meta:

        form_classname = 'waf--field'
        icon = 'extraicons--basic-field'



from wagtail import blocks

from .base_field_block import BaseFieldBlock


class HiddenFieldBlock(BaseFieldBlock):

    default_value = blocks.CharBlock(
        max_length=255,
        required=True,
        label='Value',
        help_text='The value to go into the hidden field.'
    )

    choices = None

    required = None

    help_text = None

    html = None

    display_checkbox_label = None

    empty_label = None

    max_length = None

    display_side_by_side = None

    placeholder = None

    rules = None

    class Meta:
        form_classname = 'waf--field'
        icon = 'extraicons--hidden'




# Generated by Django 3.2 on 2023-04-27 11:08

from django.db import migrations, models
import wagtail.blocks
import wagtail.contrib.forms.models
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_advanced_form_builder', '0002_auto_20201116_2241'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailformfield',
            name='choices',
            field=models.TextField(blank=True, help_text='Comma or new line separated list of choices. Only applicable in checkboxes, radio and dropdown.', verbose_name='choices'),
        ),
        migrations.AlterField(
            model_name='emailformfield',
            name='default_value',
            field=models.TextField(blank=True, help_text='Default value. Comma or new line separated values supported for checkboxes.', verbose_name='default value'),
        ),
        migrations.AlterField(
            model_name='emailformpage',
            name='form',
            field=wagtail.fields.StreamField([('singleline', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('max_length', wagtail.blocks.IntegerBlock(help_text='Set a maximum length for this field. e.g. 100', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Basic field')), ('email', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('max_length', wagtail.blocks.IntegerBlock(help_text='Set a maximum length for this field. e.g. 100', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Email field')), ('html', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='A unique reference for this html block', label='Unique reference name', max_length=255, required=True)), ('html', wagtail.blocks.RichTextBlock(required=True)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='HTML')), ('dropdown', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('empty_label', wagtail.blocks.CharBlock(max_length=255, required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Dropdown field')), ('radio', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('display_side_by_side', wagtail.blocks.BooleanBlock(help_text='Display these items side by side?', required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Radio field')), ('checkboxes', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('default_value', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(help_text='Enter a value that matches a checkbox choice above to have it ticked by default.', icon='extraicons--heading-icon', required=False))), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('display_side_by_side', wagtail.blocks.BooleanBlock(help_text='Display these items side by side?', required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Checkboxes field')), ('checkbox', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('display_checkbox_label', wagtail.blocks.BooleanBlock(default=False, help_text='Do you want the checkbox label to display? If not you should populate help_text.', required=False)), ('help_text', wagtail.blocks.CharBlock(max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Checkbox field')), ('multiselect', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Multiselect field')), ('multiline', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Multiline field')), ('url', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('max_length', wagtail.blocks.IntegerBlock(help_text='Set a maximum length for this field. e.g. 100', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='URL field')), ('hidden', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('default_value', wagtail.blocks.CharBlock(help_text='The value to go into the hidden field.', label='Value', max_length=255, required=True))], label='Hidden field')), ('number', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Number field'))], default=None, help_text='Add fields to the form', null=True, use_json_field=True),
        ),
        migrations.AlterField(
            model_name='emailformpage',
            name='from_address',
            field=models.EmailField(blank=True, max_length=255, verbose_name='from address'),
        ),
        migrations.AlterField(
            model_name='emailformpage',
            name='to_address',
            field=models.CharField(blank=True, help_text='Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.', max_length=255, validators=[wagtail.contrib.forms.models.validate_to_address], verbose_name='to address'),
        ),
        migrations.AlterField(
            model_name='formfield',
            name='choices',
            field=models.TextField(blank=True, help_text='Comma or new line separated list of choices. Only applicable in checkboxes, radio and dropdown.', verbose_name='choices'),
        ),
        migrations.AlterField(
            model_name='formfield',
            name='default_value',
            field=models.TextField(blank=True, help_text='Default value. Comma or new line separated values supported for checkboxes.', verbose_name='default value'),
        ),
        migrations.AlterField(
            model_name='formpage',
            name='form',
            field=wagtail.fields.StreamField([('singleline', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('max_length', wagtail.blocks.IntegerBlock(help_text='Set a maximum length for this field. e.g. 100', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Basic field')), ('email', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('max_length', wagtail.blocks.IntegerBlock(help_text='Set a maximum length for this field. e.g. 100', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Email field')), ('html', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='A unique reference for this html block', label='Unique reference name', max_length=255, required=True)), ('html', wagtail.blocks.RichTextBlock(required=True)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='HTML')), ('dropdown', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('empty_label', wagtail.blocks.CharBlock(max_length=255, required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Dropdown field')), ('radio', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('display_side_by_side', wagtail.blocks.BooleanBlock(help_text='Display these items side by side?', required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Radio field')), ('checkboxes', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('default_value', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(help_text='Enter a value that matches a checkbox choice above to have it ticked by default.', icon='extraicons--heading-icon', required=False))), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('display_side_by_side', wagtail.blocks.BooleanBlock(help_text='Display these items side by side?', required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Checkboxes field')), ('checkbox', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('display_checkbox_label', wagtail.blocks.BooleanBlock(default=False, help_text='Do you want the checkbox label to display? If not you should populate help_text.', required=False)), ('help_text', wagtail.blocks.CharBlock(max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Checkbox field')), ('multiselect', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('choices', wagtail.blocks.ListBlock(wagtail.blocks.TextBlock(icon='extraicons--heading-icon', required=True))), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Multiselect field')), ('multiline', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Multiline field')), ('url', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('max_length', wagtail.blocks.IntegerBlock(help_text='Set a maximum length for this field. e.g. 100', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='URL field')), ('hidden', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('default_value', wagtail.blocks.CharBlock(help_text='The value to go into the hidden field.', label='Value', max_length=255, required=True))], label='Hidden field')), ('number', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(help_text='The label of the form field.', max_length=255, required=True)), ('required', wagtail.blocks.BooleanBlock(default=False, help_text='Tick this box to make this a required field.', required=False)), ('default_value', wagtail.blocks.CharBlock(help_text='Set a default value for this field.', max_length=255, required=False)), ('help_text', wagtail.blocks.CharBlock(help_text='Text to assist the user in populating this field.', max_length=255, required=False)), ('rules', wagtail.blocks.StructBlock([('action', wagtail.blocks.ChoiceBlock(choices=[('show', 'Show'), ('hide', 'Hide')], help_text='What conditional action would you like to perform on this field?')), ('conditions', wagtail.blocks.StreamBlock([('condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')])), ('value', wagtail.blocks.CharBlock(help_text='', max_length=255, required=True))])), ('blank_condition', wagtail.blocks.StructBlock([('field_name', wagtail.blocks.CharBlock(help_text='The name of a field in the form', max_length=255, required=True)), ('rule', wagtail.blocks.ChoiceBlock(choices=[('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]))]))], required=False))], help_text='Add conditional rules to show or hide fields depending on the value of other fields in the form.', required=False))], label='Number field'))], default=None, help_text='Add fields to the form', null=True, use_json_field=True),
        ),
    ]



from django.utils.html import format_html_join
from wagtail import hooks
from wagtail.admin.staticfiles import versioned_static


@hooks.register('insert_editor_js')
def editor_js():
    js_files = [
        'wagtail_advanced_form_builder/js/admin.js', # https://fireworks.js.org
    ]
    js_includes = format_html_join('\n', '<script src="{0}"></script>',
        ((versioned_static(filename),) for filename in js_files)
    )
    return js_includes



from django import forms

from wagtail.contrib.forms.forms import FormBuilder

from wagtail_advanced_form_builder.forms.widgets.checkbox_input_widget import CheckboxInput
from wagtail_advanced_form_builder.forms.widgets.html_output_widget import HTMLOutputWidget
from wagtail_advanced_form_builder.forms.widgets.side_by_side_checkbox_select_multiple_widget import \
    SideBySideCheckboxSelectWidget
from wagtail_advanced_form_builder.forms.widgets.side_by_side_radio_select_widget import SideBySideRadioSelectWidget


class AdvancedFormBuilder(FormBuilder):

    def create_singleline_field(self, field, options):
        if field.max_length:
            options['max_length'] = field.max_length
        else:
            options['max_length'] = 255
        return forms.CharField(**options)

    def create_email_field(self, field, options):
        if field.max_length:
            options['max_length'] = field.max_length
        return forms.EmailField(**options)

    def create_url_field(self, field, options):
        if field.max_length:
            options['max_length'] = field.max_length
        return forms.URLField(**options)

    def create_html_field(self, field, options):
        options['widget'] = HTMLOutputWidget(
            html_value=field.html_value,
        )
        return forms.Field(**options)

    def create_dropdown_field(self, field, options):
        options['choices'] = list(map(
            lambda x: (x.get('value', '').strip(), x.get('value', '').strip()),
            field.choices
        ))
        if field.empty_label:
            options['choices'] = [('', field.empty_label)] + options['choices']

        return forms.ChoiceField(**options)

    def create_checkboxes_field(self, field, options):
        options['choices'] = list(map(
            lambda x: (x.get('value', '').strip(), x.get('value', '').strip()),
            field.choices
        ))
        options['initial'] = list(map(
            lambda x: x.get('value', '').strip(),
            field.default_value
        ))
        if field.display_side_by_side:
            return forms.MultipleChoiceField(
                widget=SideBySideCheckboxSelectWidget, **options
            )
        else:
            return forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple, **options
            )

    def create_checkbox_field(self, field, options):
        options['widget'] = CheckboxInput(
            display_checkbox_label=field.display_checkbox_label
        )
        return forms.BooleanField(**options)


    def create_radio_field(self, field, options):

        options['choices'] = list(map(
            lambda x: (x.get('value', '').strip(), x.get('value', '').strip()),
            field.choices
        ))

        if field.display_side_by_side:
            return forms.ChoiceField(widget=SideBySideRadioSelectWidget, **options)
        else:
            return forms.ChoiceField(widget=forms.RadioSelect, **options)


    def create_multiselect_field(self, field, options):
        options['choices'] = list(map(
            lambda x: (x.get('value', '').strip(), x.get('value', '').strip()),
            field.choices
        ))
        return forms.MultipleChoiceField(**options)





from wagtail import blocks

from .base_field_block import BaseFieldBlock


class HTMLFieldBlock(BaseFieldBlock):

    label = blocks.CharBlock(
        max_length=255,
        required=True,
        help_text='A unique reference for this html block',
        label='Unique reference name',
    )

    html = blocks.RichTextBlock(
        required=True,
    )

    help_text = None

    max_length = None

    required = None

    default_value = None

    placeholder = None

    choices = None

    empty_label = None

    display_side_by_side = None

    display_checkbox_label = None

    class Meta:
        form_classname = 'waf--field waf--html-field'
        icon = 'extraicons--paragraph'





from django.db import models

from modelcluster.fields import ParentalKey

from .abstract_advanced_form_field import AbstractAdvancedFormField


class EmailFormField(AbstractAdvancedFormField):

    # Add explicit `id` primary key using Django's defaults for versions
    # prior to 3.2 to avoid migrations when upgrading to that version.
    id = models.AutoField(
        primary_key=True,
        auto_created=True,
        serialize=False,
        verbose_name='ID',
    )
    page = ParentalKey(
        'wagtail_advanced_form_builder.EmailFormPage',
        on_delete=models.CASCADE,
        related_name='form_fields'
    )



from django.db import models
from django.utils.translation import gettext_lazy as _

from modelcluster.models import ClusterableModel

from wagtail.contrib.forms.models import AbstractFormField
from wagtail.fields import RichTextField

import wagtail_advanced_form_builder.constants as consts
from wagtail_advanced_form_builder.utils import clean_form_field_name

OVERRIDE_FORM_FIELD_CHOICES = (
    (consts.FIELD_TYPE_SINGLE_LINE, _('Single line text')),
    (consts.FIELD_TYPE_MULTI_LINE, _('Multi-line text')),
    (consts.FIELD_TYPE_EMAIL, _('Email')),
    (consts.FIELD_TYPE_NUMBER, _('Number')),
    (consts.FIELD_TYPE_URL, _('URL')),
    (consts.FIELD_TYPE_CHECKBOX, _('Checkbox')),
    (consts.FIELD_TYPE_CHECKBOXES, _('Checkboxes')),
    (consts.FIELD_TYPE_DROPDOWN, _('Drop down')),
    (consts.FIELD_TYPE_MULTI_SELECT, _('Multiple select')),
    (consts.FIELD_TYPE_RADIO, _('Radio buttons')),
    (consts.FIELD_TYPE_DATE, _('Date')),
    (consts.FIELD_TYPE_HIDDEN, _('Hidden field')),
    (consts.FIELD_TYPE_CURRENCY, _('Currency field')),
    (consts.FIELD_TYPE_SIMPLE_DATE, _('Date')),
    (consts.FIELD_TYPE_PHONE, _('Phone')),
)


class AbstractAdvancedFormField(AbstractFormField, ClusterableModel):

    # extend the built in field type choices so we can add custom fields
    CHOICES = OVERRIDE_FORM_FIELD_CHOICES

    # override the field_type field with extended choices
    field_type = models.CharField(
        verbose_name='field type',
        max_length=16,
        # use the choices tuple defined above
        choices=CHOICES
    )

    rule_action = models.CharField(
        choices=consts.FIELD_ACTION_CHOICES,
        blank=True,
        default='',
        max_length=10,
    )


    # Storage for HTML field only
    html_value = RichTextField(
        default=None,
        null=True,
        blank=True,
    )

    # Set a maximum length on fields
    max_length = models.IntegerField(
        default=None,
        null=True,
        blank=True,
    )

    # Empty label option for choice fields only
    empty_label = models.CharField(
        max_length=255,
        default=None,
        null=True,
        blank=True,
    )

    # Storage for checkboxes and radios only
    display_side_by_side = models.BooleanField(
        default=False
    )

    display_checkbox_label = models.BooleanField(
        default=False,
    )

    @property
    def clean_name(self):
        return clean_form_field_name(self.label)

    class Meta:
        abstract = True



from django.utils.translation import gettext_lazy as _

FIELD_TYPE_SINGLE_LINE = 'singleline'
FIELD_TYPE_MULTI_LINE = 'multiline'
FIELD_TYPE_EMAIL = 'email'
FIELD_TYPE_NUMBER = 'number'
FIELD_TYPE_URL = 'url'
FIELD_TYPE_CHECKBOX = 'checkbox'
FIELD_TYPE_CHECKBOXES = 'checkboxes'
FIELD_TYPE_DROPDOWN = 'dropdown'
FIELD_TYPE_MULTI_SELECT = 'multiselect'
FIELD_TYPE_RADIO = 'radio'
FIELD_TYPE_DATE = 'date'
FIELD_TYPE_HIDDEN = 'hidden'
FIELD_TYPE_HTML = 'html'
FIELD_TYPE_CURRENCY = 'currency'
FIELD_TYPE_SIMPLE_DATE = 'simpledate'
FIELD_TYPE_PHONE = 'phone'

FIELD_RULE_IS = 'is'
FIELD_RULE_IS_NOT = 'is_not'
FIELD_RULE_GREATER_THAN = 'greater_than'
FIELD_RULE_GREATER_THAN_OR_EQUAL = 'greater_than_equal'
FIELD_RULE_LESS_THAN = 'less_than'
FIELD_RULE_LESS_THAN_OR_EQUAL = 'less_than_equal'
FIELD_RULE_CONTAINS = 'contains'
FIELD_RULE_STARTS_WITH = 'starts-with'
FIELD_RULE_ENDS_WITH = 'ends-with'
FIELD_RULE_IS_BLANK = 'is_blank'
FIELD_RULE_IS_NOT_BLANK = 'is_not_blank'

FIELD_RULE_CHOICES = (
    (FIELD_RULE_IS, _('Is equal to')),
    (FIELD_RULE_IS_NOT, _('Is not equal to')),
    (FIELD_RULE_GREATER_THAN, _('Greater than')),
    (FIELD_RULE_GREATER_THAN_OR_EQUAL, _('Greater than or equal to')),
    (FIELD_RULE_LESS_THAN, _('Less than')),
    (FIELD_RULE_LESS_THAN_OR_EQUAL, _('Less than or equal to')),
    (FIELD_RULE_CONTAINS, _('Contains')),
    (FIELD_RULE_STARTS_WITH, _('Starts with')),
    (FIELD_RULE_ENDS_WITH, _('Ends with')),
)

FIELD_RULE_CHOICES_BLANK = (
    (FIELD_RULE_IS_BLANK, _('Is blank')),
    (FIELD_RULE_IS_NOT_BLANK, _('Is not blank')),
)

FIELD_ACTION_SHOW = 'show'
FIELD_ACTION_HIDE = 'hide'
FIELD_ACTION_CHOICES = (
    (FIELD_ACTION_SHOW, _('Show')),
    (FIELD_ACTION_HIDE, _('Hide')),
)



import wagtail_advanced_form_builder.constants as consts

from wagtail import blocks


class ConditionBlock(blocks.StructBlock):

    field_name = blocks.CharBlock(
        max_length=255,
        required=True,
        help_text='The name of a field in the form'
    )

    rule = blocks.ChoiceBlock(
        choices=consts.FIELD_RULE_CHOICES,
        default=consts.FIELD_RULE_IS,
    )

    value = blocks.CharBlock(
        max_length=255,
        required=True,
        help_text=''
    )

    class Meta:
        icon = 'cog'
        form_classname = 'waf--rule-condition'




from django.apps import AppConfig


class WagtailAdvancedFormBuilderConfig(AppConfig):
    name = 'wagtail_advanced_form_builder'
    verbose_name = 'Wagtail advanced form builder'
    default_auto_field = 'django.db.models.AutoField'

    def ready(self):
        pass

